/* libdevmem. Copyright 2003 Arcom Ltd */
#ifndef __LIBDEVMEM_H__
#define __LIBDEVMEM_H__

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/* 
 * A handle for an open mapping. This structure is opaque and should
 * not be accessed directly
 */
typedef struct libdevmem_handle_t {
	uint8_t *base;
	void *mapping;
	unsigned long length;
	int fd;
} *libdevmem_handle;

/* libdevmem_open - Open a mapping to a physical address location. 
 *
 * base		The physical address to map
 * length	The length of the region to map
 *
 * returns	A handle to the mapping. On error NULL is returned and errno is set.
 */
extern libdevmem_handle libdevmem_open(unsigned long base, unsigned long length);

/* libdevmem_close - Close a mapping.
 *
 * handle	The mapping to close
 *
 * returns	0 on success. On error -1 is returned and errno is set.
 */
extern int libdevmem_close(libdevmem_handle handle);

/* libdevmem_read_uint8 - Read an unsigned 8 bit value from an offset into a mapping 
 *
 * handle	The mapping to access
 * offset	Offset into the mapping to read from
 *
 * returns	An unsigned 8 bit value
 */
static inline uint8_t libdevmem_read_uint8(libdevmem_handle handle, unsigned long offset)
{
	volatile uint8_t *reg = (volatile uint8_t *)(handle->base + offset);
	return *reg;
}

/* libdevmem_write_uint8 - Write an unsigned 8 bit value to an offset in a mapping 
 *
 * handle	The mapping to access
 * offset	Offset into the mapping to write to
 * value	The unsigned 8 bit value to write
 *
 * returns	n/a
 */
static inline void libdevmem_write_uint8(libdevmem_handle handle, unsigned long offset, uint8_t value)
{
	volatile uint8_t *reg = (volatile uint8_t *)(handle->base + offset);
	*reg = value;
}

/* libdevmem_read_uint16 - Read an unsigned 16 bit value from an offset into a mapping 
 *
 * handle	The mapping to access
 * offset	Offset into the mapping to read from
 *
 * returns	An unsigned 16 bit value
 */
static inline uint16_t libdevmem_read_uint16(libdevmem_handle handle, unsigned long offset)
{
	volatile uint16_t *reg = (volatile uint16_t *)(handle->base + offset);
	return *reg;
}

/* libdevmem_write_uint16 - Write an unsigned 16 bit value to an offset in a mapping 
 *
 * handle	The mapping to access
 * offset	Offset into the mapping to write to
 * value	The unsigned 16 bit value to write
 *
 * returns	n/a
 */
static inline void libdevmem_write_uint16(libdevmem_handle handle, unsigned long offset, uint16_t value)
{
	volatile uint16_t *reg = (volatile uint16_t *)(handle->base + offset);
	*reg = value;
}

/* libdevmem_read_uint32 - Read an unsigned 32 bit value from an offset into a mapping 
 *
 * handle	The mapping to access
 * offset	Offset into the mapping to read from
 *
 * returns	An unsigned 32 bit value
 */
static inline uint32_t libdevmem_read_uint32(libdevmem_handle handle, unsigned long offset)
{
	volatile uint32_t *reg = (volatile uint32_t *)(handle->base + offset);
	return *reg;
}

/* libdevmem_write_uint32 - Write an unsigned 32 bit value to an offset in a mapping 
 *
 * handle	The mapping to access
 * offset	Offset into the mapping to write to
 * value	The unsigned 32 bit value to write
 *
 * returns	n/a
 */
static inline void libdevmem_write_uint32(libdevmem_handle handle, unsigned long offset, uint32_t value)
{
	volatile uint32_t *reg = (volatile uint32_t *)(handle->base + offset);
	*reg = value;
}

#ifdef __cplusplus
}
#endif

#endif /* __LIBDEVMEM_H__ */
